@@ -54,6 +54,7 @@ import net.sourceforge.jnlp.config.Defaults;
 import net.sourceforge.jnlp.OptionsDefinitions;
 import net.sourceforge.jnlp.config.InfrastructureFileDescriptor;
 import net.sourceforge.jnlp.config.Setting;
+import net.sourceforge.jnlp.runtime.JNLPRuntime;
 import net.sourceforge.jnlp.runtime.Translator;
 import net.sourceforge.jnlp.util.docprovider.formatters.formatters.Formatter;
 import net.sourceforge.jnlp.util.docprovider.formatters.formatters.HtmlFormatter;
@@ -396,10 +397,13 @@ public abstract class TextsProvider {
     }
 
     public static void main(String[] args) throws IOException {
+        // Shutdown hook from OutputController was causing hanging build on Windows. It's not used on headless.
+        JNLPRuntime.setHeadless(true);
+
         if (args.length == 0) {
             System.out.println(" * IcedTea-Web self documentation tool list of arguments *");
-            System.out.println(" * arument version - last parameter of each command, is used when there is no internal versionknown *");
-            System.out.println(" *                 - is mandatory, but not used if real version do exists *");
+            System.out.println(" * argument version - last parameter of each command, is used when there is no internal versionknown *");
+            System.out.println(" *                  - is mandatory, but not used if real version do exists *");
             System.out.println(" * argument expand - one before last argument, false/true - is used to not/expand variables *");
             System.out.println(" * -------------------- *");
             System.out.println("all expand version - will generate all reports in theirs defaults into current directory");
